extends CharacterBody3D


const WALK_VELOCITY = 12.0
const RUN_VELOCITY = 24.0
const JUMP_VELOCITY = 12.0
const SENSETIVITY = 0.009;

var gravity = 9.8
var speed = WALK_VELOCITY
@onready var head = $Head
@onready var camera = $Head/Camera
@onready var playerCharacterBody = $"."
var is_playable = false
var internal_id

func set_internal_id(id):
	internal_id = id

func set_is_playable(value):
	is_playable = value

func test(text):
	print(text)

func _ready():
	if (!is_playable): return
	camera.make_current()
	playerCharacterBody.up_direction = Vector3.UP;
	Input.set_mouse_mode(Input.MOUSE_MODE_CAPTURED)

func _unhandled_input(event):
	if (!is_playable): return
	if event is InputEventMouseMotion:
		head.rotate_y(-event.relative.x * SENSETIVITY)
		camera.rotate_x(-event.relative.y * SENSETIVITY)
		camera.rotation.x = clamp(camera.rotation.x, deg_to_rad(-50), deg_to_rad(60))

func _physics_process(delta):
	if (!is_playable): return	
	if not is_on_floor():
		velocity.y -= 4 * gravity * delta

	var input_dir = Input.get_vector("left", "right", "forward", "backward")
	var direction = (head.transform.basis * Vector3(input_dir.x, 0, input_dir.y)).normalized()
	
	if is_on_floor():
		if Input.is_action_just_pressed("jump"):
			velocity.y = JUMP_VELOCITY
		if Input.is_action_pressed("run"):
			speed = RUN_VELOCITY
		else:
			speed = WALK_VELOCITY
		if direction:
			velocity.x = direction.x * speed
			velocity.z = direction.z * speed
		else:
			velocity.x = lerp(velocity.x, direction.x * speed, delta * 8)
			velocity.z = lerp(velocity.z, direction.z * speed, delta * 8)
	else:
		velocity.x = lerp(velocity.x, direction.x * speed, delta)
		velocity.z = lerp(velocity.z, direction.z * speed, delta)
	move_and_slide()
	if (!multiplayer.is_server()):
		Networking.move_client.rpc_id(1, multiplayer.get_unique_id(), $".".position)
		print("I am " + $"..".name +", belonging of " + str(multiplayer.get_unique_id()) + ", with internal ID: " + str(internal_id))

@rpc ("authority", "call_remote")
func move_puppet(p: Vector3, i_id):
	if (i_id == internal_id):
		playerCharacterBody.position = p
